@@ -1,19 +1,11 @@
 __all__ = (
-    "IsMallocPointerError",
     "AllocationError",
-    "NotEnoughChunks",
-    "IsFrozenError",
     "DereferenceError",
     "FreedMemoryError",
     "InvalidSizeError",
     "InvalidBindingParameter",
-)  # noqa
-
-
-class IsMallocPointerError(Exception):
-    """Raised when trying perform an operation on a malloc pointer that isn't supported."""  # noqa
-
-    pass
+    "NullPointerError",
+)
 
 
 class AllocationError(Exception):
@@ -22,18 +14,6 @@ class AllocationError(Exception):
     pass
 
 
-class NotEnoughChunks(Exception):
-    """Raised when there aren't enough chunks in a CallocPointer."""
-
-    pass
-
-
-class IsFrozenError(Exception):
-    """Raised when trying to move the address of a frozen pointer."""
-
-    pass
-
-
 class DereferenceError(Exception):
     """Raised when dereferencing an address fails."""  # noqa
 
@@ -53,6 +33,12 @@ class InvalidSizeError(Exception):
 
 
 class InvalidBindingParameter(Exception):
-    """Raised when an invalid type is passed to the"""
+    """Raised when an invalid type is passed to a binding."""
+
+    pass
+
+
+class NullPointerError(Exception):
+    """Raised when a pointer is null."""
 
     pass
